@@ -1,7 +1,7 @@
 from ._version import __version__
 
 from .models import (
-    BartForSequenceClassification, Blender, CoreferencePredictor, DistilBertForMaskedLM,
+    BartForSequenceClassification, Blender, DistilBertForMaskedLM,
     ElectraForSequenceClassification, RobertaForQuestionAnswering
 )
 
