@@ -113,7 +113,7 @@ class BartForSequenceClassification(Module):
             'norm3': 'final_layer_norm'
         }
 
-        for layer_ind in range(len(self.transformer.encoder.layers)):
+        for layer_ind in range(len(self.transformer.decoder.layers)):
             prefix = f'model.decoder.layers.{layer_ind}.'
             for w in ('weight', 'bias'):
                 state_dict[f'{prefix}self_attention.pre_attention.{w}'] = transposed_cat([
@@ -127,7 +127,7 @@ class BartForSequenceClassification(Module):
             'model.encoder.version', 'model.decoder.version', 'model.encoder.embed_tokens.weight',
             'model.decoder.embed_tokens.weight']
         for key in keys_to_delete:
-            del state_dict[key]
+            state_dict.pop(key, None)
         old_names = {
             'model.encoder_positions.weight': 'model.encoder.embed_positions.weight',
             'model.encoder_norm.weight': 'model.encoder.layernorm_embedding.weight',
@@ -161,11 +161,11 @@ class BartForSequenceClassification(Module):
         super().load_state_dict(state_dict, strict)
 
     @classmethod
-    def from_pretrained(cls, model_path: str) -> 'BartForSequenceClassification':
+    def from_pretrained(cls, model_path: str, decoder_layers: int = 12) -> 'BartForSequenceClassification':
         model = cls(
             vocab_size=50265, padding_idx=1, dim=1024, max_position_embeddings=1024,
             encoder_layers=12, encoder_heads=16, encoder_ffn_dim=4096,
-            decoder_layers=12, decoder_heads=16, decoder_ffn_dim=4096, num_labels=3, eos_token_id=2
+            decoder_layers=decoder_layers, decoder_heads=16, decoder_ffn_dim=4096, num_labels=3, eos_token_id=2
         )
         model.load_state_dict(state_dict=torch.load(os.path.join(model_path, 'pytorch_model.bin'), map_location='cpu'))
         model.eval()
