@@ -1,47 +1,62 @@
 from typing import Tuple
 
 import os
-import json
 import math
 
 import torch
 from torch.nn.functional import relu
 
-from .feed_forward import FeedForward
-from .gated_sum import GatedSum
-from .span_extractors import SpanExtractor, SelfAttentiveSpanExtractor
-from .transformer_embedder import TransformerEmbedder
-from .time_distributed import TimeDistributed
+from allennlp.data import Vocabulary
+from allennlp.models.model import Model
+from allennlp.modules import FeedForward, GatedSum
+from allennlp.modules import Seq2SeqEncoder, TimeDistributed, TextFieldEmbedder
+from allennlp.modules.span_extractors import SelfAttentiveSpanExtractor, EndpointSpanExtractor
+from allennlp.common import Params
+
 from .coref_utils import (
     masked_topk, flatten_and_batch_shift_indices, batched_index_select, masked_softmax,
     weighted_sum, bucket_values)
 from ...utils import neg_inf
 
 
-class CoreferencePredictor(torch.nn.Module):
+class CoreferencePredictor(Model):
     def __init__(
-            self, text_field_embedder: dict, mention_feedforward: dict, antecedent_feedforward: dict,
-            feature_size: int, max_span_width: int, spans_per_word: float, max_antecedents: int, inference_order: int
-    ):
-        super().__init__()
+            self,
+            vocab: Vocabulary,
+            text_field_embedder: TextFieldEmbedder,
+            context_layer: Seq2SeqEncoder,
+            mention_feedforward: FeedForward,
+            antecedent_feedforward: FeedForward,
+            feature_size: int,
+            max_span_width: int,
+            spans_per_word: float,
+            max_antecedents: int,
+            coarse_to_fine: bool = False,
+            inference_order: int = 1,
+            lexical_dropout: float = 0.2,
+            **kwargs
+    ) -> None:
+        super().__init__(vocab, **kwargs)
 
-        self.transformer = TransformerEmbedder(
-            max_length=text_field_embedder['token_embedders']['tokens']['max_length'])
-        self._mention_feedforward = TimeDistributed(FeedForward(
-            input_dim=mention_feedforward['input_dim'], num_layers=mention_feedforward['num_layers'],
-            dims=mention_feedforward['hidden_dims'])
+        self._text_field_embedder = text_field_embedder
+        self._context_layer = context_layer
+        self._mention_feedforward = TimeDistributed(mention_feedforward)
+        self._mention_scorer = TimeDistributed(
+            torch.nn.Linear(mention_feedforward.get_output_dim(), 1)
         )
-        self._mention_scorer = TimeDistributed(torch.nn.Linear(self._mention_feedforward.get_output_dim(), 1))
-
-        self._antecedent_feedforward = TimeDistributed(FeedForward(
-            input_dim=antecedent_feedforward['input_dim'], num_layers=antecedent_feedforward['num_layers'],
-            dims=antecedent_feedforward['hidden_dims'])
+        self._antecedent_feedforward = TimeDistributed(antecedent_feedforward)
+        self._antecedent_scorer = TimeDistributed(
+            torch.nn.Linear(antecedent_feedforward.get_output_dim(), 1)
         )
-        self._antecedent_scorer = TimeDistributed(torch.nn.Linear(self._antecedent_feedforward.get_output_dim(), 1))
 
-        self._endpoint_span_extractor = SpanExtractor(
-            num_width_embeddings=max_span_width, span_width_embedding_dim=feature_size)
-        self._attentive_span_extractor = SelfAttentiveSpanExtractor(input_dim=self.transformer.get_output_dim())
+        self._endpoint_span_extractor = EndpointSpanExtractor(
+            context_layer.get_output_dim(),
+            combination="x,y",
+            num_width_embeddings=max_span_width,
+            span_width_embedding_dim=feature_size,
+            bucket_widths=False,
+        )
+        self._attentive_span_extractor = SelfAttentiveSpanExtractor(input_dim=text_field_embedder.get_output_dim())
 
         # 10 possible distance buckets.
         self._num_distance_buckets = 10
@@ -52,55 +67,18 @@ class CoreferencePredictor(torch.nn.Module):
         self._max_antecedents = max_antecedents
 
         self._coarse2fine_scorer = torch.nn.Linear(
-            self._mention_feedforward.get_input_dim(), self._mention_feedforward.get_input_dim())
+            mention_feedforward.get_input_dim(), mention_feedforward.get_input_dim())
         self._inference_order = inference_order
-        self._span_updating_gated_sum = GatedSum(self._mention_feedforward.get_input_dim())
+        self._span_updating_gated_sum = GatedSum(mention_feedforward.get_input_dim())
 
         self.pad_map = {'spans': -1}
 
-    def load_state_dict(self, state_dict, strict=True):
-        module_names = {
-            'layers': 'layer',
-            'transformer.model': '_text_field_embedder.token_embedder_tokens._matched_embedder.transformer_model'
-        }
-        for key, value in list(state_dict.items()):
-            new_key = key
-            for name, old_name in module_names.items():
-                new_key = new_key.replace(f'{old_name}.', f'{name}.')
-            state_dict[new_key] = state_dict.pop(key)
-        del state_dict['transformer.model.embeddings.position_ids']
-        del state_dict['transformer.model.pooler.dense.weight']
-        del state_dict['transformer.model.pooler.dense.bias']
-        old_names = {
-            'attention.out_lin': 'attention.output.dense',
-            'norm1': 'attention.output.LayerNorm',
-            'norm2': 'output.LayerNorm',
-            'ffn.lin1': 'intermediate.dense',
-            'ffn.lin2': 'output.dense'
-        }
-        state_dict['transformer.model.embeddings.word_embeddings.weight'] += state_dict.pop(
-            'transformer.model.embeddings.token_type_embeddings.weight')[0]
-
-        scale = math.sqrt(self.transformer.model.encoder.layers[0].attention.dim_per_head)
-
-        for layer_ind in range(len(self.transformer.model.encoder.layers)):
-            prefix = f'transformer.model.encoder.layers.{layer_ind}.'
-            for p in ('weight', 'bias'):
-                state_dict[f'{prefix}attention.self.query.{p}'] /= scale
-                state_dict[f'{prefix}attention.pre_attention.{p}'] = torch.cat([
-                    state_dict.pop(f'{prefix}attention.self.{k}.{p}') for k in ('query', 'key', 'value')
-                ], dim=0)
-                for new_name, old_name in old_names.items():
-                    state_dict[f'{prefix}{new_name}.{p}'] = state_dict.pop(f'{prefix}{old_name}.{p}')
-        return super().load_state_dict(state_dict, strict)
-
     @classmethod
     def from_pretrained(cls, model_path: str):
-        with open(os.path.join(model_path, 'config.json'), 'r') as file:
-            model_params = json.load(file)['model']
-        for key in ('type', 'initializer', 'coarse_to_fine', 'context_layer'):
+        model_params = Params.from_file(os.path.join(model_path, 'config.json'))['model']
+        for key in ('type', 'initializer'):
             del model_params[key]
-        model = cls(**model_params)
+        model = cls.from_params(vocab=None, params=model_params, serialization_dir=model_path)
         model.load_state_dict(torch.load(os.path.join(model_path, 'weights.th'), map_location='cpu'), strict=True)
         model.eval()
         for p in model.parameters():
@@ -108,7 +86,13 @@ class CoreferencePredictor(torch.nn.Module):
         return model
 
     def forward(self, spans, token_ids, mask, offsets, segment_concat_mask):
-        text_embeddings = self.transformer(token_ids=token_ids, offsets=offsets, segment_concat_mask=segment_concat_mask)
+        text = {'tokens': {
+            'token_ids': token_ids, 'mask': mask, 'type_ids': None, 'offsets': offsets,
+            'wordpiece_mask': None, 'segment_concat_mask': segment_concat_mask
+        }}
+
+        # Shape: (batch_size, document_length, embedding_size)
+        text_embeddings = self._text_field_embedder(text)
 
         batch_size, num_spans = spans.shape[:2]
         document_length = text_embeddings.shape[1]
