@@ -2,7 +2,6 @@ from typing import Optional
 
 import os
 import logging
-import math
 
 import torch
 from torch import Tensor
@@ -10,6 +9,7 @@ from torch.nn import Module
 from torch.nn.functional import gelu
 
 from ..modules import Linear, TransformerEncoder, EmbeddingsWithTokenTypes
+from ..utils import transposed_cat
 
 
 logger = logging.getLogger(__name__)
@@ -97,16 +97,12 @@ class ElectraForSequenceClassification(Module):
             'ffn.lin1': 'intermediate.dense',
             'ffn.lin2': 'output.dense'
         }
-
-        scale = math.sqrt(self.transformer.encoder.layers[0].attention.dim_per_head)
-
         for layer_ind in range(len(self.transformer.encoder.layers)):
             prefix = f'transformer.encoder.layers.{layer_ind}.'
             for p in ('weight', 'bias'):
-                state_dict[f'{prefix}attention.self.query.{p}'] /= scale
-                state_dict[f'{prefix}attention.pre_attention.{p}'] = torch.cat([
+                state_dict[f'{prefix}attention.pre_attention.{p}'] = transposed_cat([
                     state_dict.pop(f'{prefix}attention.self.{k}.{p}') for k in ('query', 'key', 'value')
-                ], dim=0)
+                ])
                 for new_name, old_name in old_names.items():
                     state_dict[f'{prefix}{new_name}.{p}'] = state_dict.pop(f'{prefix}{old_name}.{p}')
         return super().load_state_dict(state_dict, strict)
