@@ -113,14 +113,18 @@ class ClientAPI:
             }
         return cls.concat_lists_or_tensors(outputs)
 
-    @staticmethod
-    def clear_sockets(socket_list: List[ClientSocket]):
+    def clear_sockets(self, socket_list: List[ClientSocket]):
         while True:
             active_sockets: List[ClientSocket] = select.select(socket_list, [], [], 0)[0]
             if not active_sockets:
                 break
             for client_socket in active_sockets:
-                client_socket.connection.recv(client_socket.message_size)
+                data = client_socket.connection.recv(client_socket.message_size)
+                if not data:
+                    socket_list.remove(client_socket)
+                    socket_list.extend(self.create_sockets(
+                        self.model, self.num_models, self.port, attempts=120, wait_time=10
+                    ))
 
     def forward(self, **inputs):
         thread_id = threading.get_ident().to_bytes(length=Config.thread_id_len, byteorder='big', signed=False)
